-- Launcher for R4Y render farm tool: check for update/install and then run the main script
-- (c) Martin Breidt - 11.06.2015 - martin@breidt.net

/*
macroscripts end up in 	getDir #userMacros

#userMacros		"C:\Users\user\AppData\Local\Autodesk\3dsMax\2014 - 64bit\ENU\usermacros"
#userScripts 	"C:\Users\user\AppData\Local\Autodesk\3dsMax\2014 - 64bit\ENU\scripts"
#scripts 			"C:\Program Files\Autodesk\3ds Max 2014\scripts"
#temp 			"C:\Users\user\AppData\Local\Autodesk\3dsMax\2014 - 64bit\ENU\temp"
*/

macroscript runFarmTool 
	category:"R4Y Tools" 
	buttonText:"Submit to render4you"
	tooltip:"Open R4Y render farm tool for render job submission"
(
	global mbVerbFlag = true						-- set to TRUE for additional debug information in the MXS listener
	
	local mainTitle = "farmTool Launcher"		--  title of this tool
	local appVersion									-- version number for comparison in update check
	local fullTitle = mainTitle + " " + (appVersion as string)

	local updFname = "version.ini"											-- CONFIG: name of the online .ini file to retrieve when checking for update
	local updURL = "http://wp1021807.server-he.de/interface/plugin/3dsmax/" + updFname		-- CONFIG: Complete URL for update check

	local updSection = "update"												-- CONFIG: section name in downloaded .ini file
	local updVersionKey = "version"											-- CONFIG: key for version value in .ini file
	local updURLKey = "url"													-- CONFIG: key for download URL value in .ini file - this is the acutal update file being downloaded
	
	local currentScriptFName = "goFarming"								-- CONFIG: name of the MAIN script file in #userScripts - this file will be optionally updated and run at the end
	
	-------------- functions
	
	fn popupQuestion msg title: = (		-- ask yes/no msg and return answer
		if title==unsupplied then title="Question"
		queryBox msg title:(fullTitle + " - " + title) beep:true  
	)
	
	fn popupMsg msg title: = (	-- show message window
		if title==unsupplied then title="Message"
		messageBox msg title:(fullTitle + " - " + title) beep:true  
	)	
	
	fn getFileFromURL URL fName = (	-- download file from URL to local file fName; returns true on success, false otherwise
		local webUri = dotNetObject "System.Uri" URL		-- convert URL string to Uri object
		local webClient = dotNetObject "System.Net.WebClient"
		local res = undefined
	
		progressStart "Downloading file..."
		try (
			webClient.DownloadFile webUri fName
			res = true
		) catch (
			-- catch error message from webClient and parse into useful text
			local ce = getCurrentException()
			local i = findString ce "exception:"
			if i!=undefined then ce = subString ce (i+11) -1
			popupMsg ("Error downloading " + URL + " to '" + fName + "':\n\n" + ce)
			res = false
		)
		progressEnd()
		
		res
	)
	
	fn newVersionAvailable = (	-- check online for a newer version and return the URL for the new file or "" if no newer version is available; returns undefined on error
		local retVal = undefined
		
		try (
			local tempFileName = sysInfo.tempdir + updFname
			if mbVerbFlag then format "tempFileName = %\n" tempFileName
			
			local res = getFileFromURL updURL tempFileName		-- download INI file to temp folder
			
			if res then (
				if mbVerbFlag then format "successfully downloaded .ini file from %\n" updURL
				-- parse INI file for version number and download URL
				local onlineVerStr = getINISetting tempFileName updSection updVersionKey	-- get version number
				local updateURL = getINISetting tempFileName updSection updURLKey			-- get download URL
				if mbVerbFlag then format "onlineVerStr = '%'    updateURL = '%'\n" onlineVerStr updateURL
				
				if onlineVerStr!="" and updateURL!="" then (
					local onlineVer = (onlineVerStr as float)	-- ... then convert the string to float
					if mbVerbFlag then format "Final onlineVer = %\n" onlineVer
					if ((classOf onlineVer)!=Float) then (
						format "Error while parsing online version number! (%)\n" onlineVerStr
						popupMsg ("Error while parsing online version number! (" + onlineVerStr + ")") title:"Error"
					) else (
						retVal = #(onlineVer, updateURL)
					)
				) else (
					popupMsg "Error while parsing online update information!" title:"Error"
				)
				deleteFile tempFileName			-- clean up the downloaded INI file
			) else (
				popupMsg "Error downloading online update information from " + updURL +  " to " + tempFileName + "!" title:"Error"
			)
		) catch (
			popupMsg ("Error while checking online for new version:\n\n" + (getCurrentException()))
		)
		retval
	)
	
	fn getLatestLocalScript baseStr = (		-- search for all files matching baseStr and return the last one after sorting
		local allFiles = getFiles (baseStr + "*")
		local res = baseStr
		if allFiles.count > 0 then (
			sort allFiles
			res = allFiles[allFiles.count]
		)
		res
	)
	
	fn updateNewVersion = (	-- download new script version if there is one; return name of the downloaded script or "" if nothing was downloaded
		
--		local currentScriptPath = ( (getDir #userScripts) + "\\" + currentScriptFName )		-- full path of the existing main script, to be replaced by the update
		local currentScriptPath = getLatestLocalScript ((getDir #userScripts) + "\\" + currentScriptFName )		
		if mbVerbFlag then format "updateNewVersion: currentScriptPath = '%'\n" currentScriptPath

		-- extract app version from filename (format: base_XXXXX.ext)
		local tkn = filterString (getFilenameFile currentScriptPath) "_"
		if tkn.count==2 then (
			local appVersionInt = tkn[2] as integer
			if mbVerbFlag then format "appVersionInt = %\n" appVersionInt
			if classOf(appVersionInt) == integer then (
				appVersion = appVersionInt/1000.0
			) else (
				appVersion = 0.0
			)
		) else (
			appVersion = 0.0		-- nothing useful found
		)
		if mbVerbFlag then format "local appVersion = %\n" appVersion

		local newSFname = currentScriptPath		-- store path of old script file, in case the updating fails

		local onlineVer = newVersionAvailable()		-- go online and check for new version using INI file
		if mbVerbFlag then format "updateNewVersion: onlineVer = %\n" onlineVer
		
		if (onlineVer != undefined) and ((classOf onlineVer)==Array) and (onlineVer.count>0) then (
			-- we have a valid result from the update check
			if mbVerbFlag then format "updateNewVersion: onlineVer[1] = %   appVersion = %\n" onlineVer[1] appVersion

			local fullInstall = false
			
			if not doesFileExist currentScriptPath then (
				if (popupQuestion ("No local installation of main farm script found (" + currentScriptPath + ")\n\nDo you want to download and install the latest version?")) then (
					fullInstall = true
					appVersion = 0.0
				) else (
					popupMsg "No installation found.\n\nExiting now..." title:"Error"
					return ""		-- return empty script file name
				)
			)
			
			local tempFileName = sysInfo.tempdir + (filenameFromPath onlineVer[2])
			if mbVerbFlag then format "updateNewVersion: tempFileName = '%'\n" tempFileName
			local newScriptPath = (getFilenamePath currentScriptPath) + (getFilenameFile tempFileName) + (getFilenameType tempFileName)		-- make sure the new script is using the old filename but the new extension!

			if onlineVer[1] > appVersion then	(
				-- update check found a NEWER version available online
				if mbVerbFlag then format "updateNewVersion: Found newer version online\n"
				if fullInstall or (popupQuestion ("You are currently using script version " + (appVersion as string) + "\n\nA new version " + (onlineVer[1] as string) + " is available at " + onlineVer[2] + "\n\nWould you like to download and install the new version?")) then (
					if fullInstall then format "Doing full installation from download\n"
					local res = getFileFromURL onlineVer[2] tempFileName		-- DOWNLOAD the newer file to temp folder
					
					if res then (
						-- download successful, now replace old script with new one
						if mbVerbFlag then format "Downloaded update to '%', will copy to '%'\n" tempFileName newScriptPath

						-- copy downloaded script to correct place and restart
						if fullInstall then (
							-- just copy the downloaded script to the default path
							if not (copyFile tempFileName newScriptPath) then (
								popupMsg ("Could not install the downloaded file from '" + tempFileName + "' to '" + newScriptPath + "'.\n\nExiting now...") title:"Fatal Error"
								return ""
							) else (
								popupMsg ("Successfully downloaded and installed version " + (onlineVer[1] as string) + " to " + newScriptPath + "\n\nStarting tool now..." ) title:"Success"
								newSFname = newScriptPath
							)
						) else (
							-- normal update from existing file
							if not (deleteFile newScriptPath) then format "Could not delete '%'\n" newScriptPath
							if not (copyFile tempFileName newScriptPath) then (
								if mbVerbFlag then format "copyFile \"%\" \"%\" failed\n" tempFileName newScriptPath
								popupMsg "Could not copy downloaded file to script folder!" title:"Error"
							) else (
								popupMsg ("Successfully updated to version " + (onlineVer[1] as string) + " to " + newScriptPath + "\n\nStarting tool now...") title:"Success"
								if mbVerbFlag then (
									format "Copy of '%' to '%' successful\n" tempFileName newScriptPath
									format "--> New version of script has been installed.\n"
								)
								newSFname = newScriptPath
							)
						)
						deleteFile tempFileName		-- clean up downloaded file from temp folder
					) else (
						-- failed to download file from URL
						popupMsg ("Could not download update from " + onlineVer[2] + " to " + tempFileName + "\n\nWill continue with current version.") title:"Error"
						if mbVerbFlag then format ("Could not download update from '" + onlineVer[2] + "' to '" + tempFileName + "'\n")
					)
				) else (
					if mbVerbFlag then format "updateNewVersion: Update cancelled by user\n"
				)
			) else (
				if mbVerbFlag then format "updateNewVersion: No newer version found\n"
			)
		) else popupMsg "Could not check for online update.\n\nWill continue with current version." title:"Error"
		
		newSFname	-- return main script file name
	)
	
	on execute do (
		------------- main body
		clearListener()
		if mbVerbFlag then format "Verbose mode.\n"
		if mbVerbFlag then format "Checking for update...\n"
		-- if update available, then download update and replace old .mse file with new one
		local scriptToRun = updateNewVersion()

		-- run main code from .mse 
		if (doesFileExist scriptToRun) then (
			if mbVerbFlag then format "Opening main farm tool now...\n"
			fileIn scriptToRun
		) else (
			if scriptToRun.count > 0 then popupMsg ("Could not find main script file '" + scriptToRun + "' - Existing now!") title:"Fatal Error"
		)
	)
)
------------------------------------------------------- end of macroscript code

---- install new entry at the end of the render menu if not done before	== THIS CODE WILL RUN IMMEDIATELY UPON EVALUATION OF THIS FILE!
if (queryBox "Do you want to add a new entry to the main Rendering menu for submitting to the render farm?" title:"Render Farm Installation" beep:true ) then (
	format "Checking menu entry...\n"
	if menuMan.registerMenuContext 0x50e4691f then (		-- magic number used for this specific menu entry - DO NOT CHANGE UNLESS YOU WANT DUPLICATE ENTRIES!
		-- we have not registered our new entry on this installation of 3ds Max yet, so let's do that now...
		format "Adding new entry to the Rendering menu...\n"
		local mainMenuBar = menuMan.getMainMenuBar()		-- Get the main menu bar
		local mmnItems = mainMenuBar.numItems()				-- how many entries in the main menu
		format "Found % entries in the main menu\n" mmnItems
		local success = false
		for m = 1 to mmnItems do (
			-- search through all entries of the main menu for the Rendering menu
			local mItem = mainMenuBar.getItem(m)
			local iTitle = mItem.getTitle()
			format "Checking submenu #% '%'\n" m iTitle
			if (matchPattern iTitle pattern:"*&R*") then (
				-- found &R key accelerator in menu title --> assume this is the Rendering menu, let's add a new entry
				format "Found rendering menu at index %: '%'\n" m iTitle
				-- local sepItem = menuMan.createSeparatorItem()
				local theAction = menuMan.createActionItem "runFarmTool" "R4Y Tools" 	-- create an ActionItem from the MacroScript defined above
				
				if theAction!=undefined then (
					local renderMenu = mItem.getSubMenu()
					-- add separator and new menu entry to the end of the menu
					-- renderMenu.addItem sepItem -1		
					-- renderMenu.addItem theAction -1
					renderMenu.addItem theAction 2			-- add entry at position 2
					format "Added new entry to the Render menu at position 2.\n"
					
					menuMan.updateMenuBar()		-- update the menu bar 
					format "Done adding render farm entry to the Render menu\n"
					success = true
					exit		-- exit for loop
				) else (
					format "Error creating the actionItem for the Rendering menu! - Is macroscript installed correctly?\n"
				)
			)
		)
		format "Done searching the main menu.\n"
		if success then (
			messageBox "Successfully added a new entry for render farm submission at the end of the 'Render' main menu." title:"Information" beep:false
		) else (
			messageBox "Could not add a new entry to the main 'Rendering' menu. Maybe the menu is missing or has been renamed?" title:"Error" beep:true
		)
	) else messageBox "Menu entry already present, not doing any changes to the menu." title:"Information" beep:false
) else (
	format "No changes made to the main menu.\n"
	messageBox "No changes made to menu. Use the 'R4Y Tools' category in Customize UI dialog to install macroscript into UI" title:"Information" beep:false
)
